package de.mdelab.mlsdm.interpreter.debug.ui.breakpoints;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.model.IBreakpoint;
import org.eclipse.emf.ecore.provider.EcoreItemProviderAdapterFactory;
import org.eclipse.emf.edit.ui.provider.AdapterFactoryLabelProvider;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.handlers.HandlerUtil;

import de.mdelab.mlcore.MLElementWithUUID;
import de.mdelab.mlstorypatterns.diagram.custom.edit.parts.IMLExpressionEditPart;
import de.mdelab.sdm.interpreter.code.debug.ui.breakpoints.SDDebugBreakpoint;
import de.mdelab.sdm.interpreter.code.debug.ui.storyDiagramEditor.SDEditorManager;

/**
 * @see org.eclipse.core.commands.IHandler
 * @see org.eclipse.core.commands.AbstractHandler
 */
public class ToggleBreakpointsHandler extends AbstractHandler {
	
	private final AdapterFactoryLabelProvider emfLabelProvider;

    /**
	 * The constructor.
	 */
	public ToggleBreakpointsHandler() {
		super();
		
		emfLabelProvider = new AdapterFactoryLabelProvider( new EcoreItemProviderAdapterFactory() );
	}

	@Override
	public Object execute( final ExecutionEvent p_event )
	throws ExecutionException {
		final List<MLElementWithUUID> elements = new ArrayList<MLElementWithUUID>();
	
		final ISelection currentSelection = HandlerUtil.getCurrentSelection( p_event );

		if ( currentSelection instanceof IStructuredSelection ) {
			for (final Object element : ( (IStructuredSelection) currentSelection ).toList() ) {
				final MLElementWithUUID uuidElement;
				
				if ( element instanceof IMLExpressionEditPart ) {
					uuidElement = ( (IMLExpressionEditPart) element ).getMLExpression();
				}
				else if ( element instanceof IAdaptable ) {
					final IAdaptable adaptable = (IAdaptable) element;
					uuidElement = (MLElementWithUUID) adaptable.getAdapter( MLElementWithUUID.class );
				}
				else {
					uuidElement = null;
				}
				
				if ( uuidElement != null ) {
					elements.add( uuidElement );
				}
			}
			
			for ( final MLElementWithUUID element : elements ) {
				final String elementUUID = element.getUuid();
				
				IBreakpoint breakpoint = SDDebugBreakpoint.findBreakpoint( elementUUID );

				try	{
					if ( breakpoint == null ) {
						final String elementName = emfLabelProvider.getText( element );
						
//						if ( element instanceof MLNamedElement ) {
//							elementName = ( (MLNamedElement) element ).getName() + " : " + element.eClass().getName();
//						}
//						else {
//							final EObject container = element.eContainer();
//							final StringBuilder strBuild = new StringBuilder();
//							
//							if ( container instanceof MLNamedElement ) {
//								strBuild.append( ( (MLNamedElement) container ).getName() );
//								strBuild.append( "." );
//							}
//							
//							strBuild.append( element.eClass().getName() );
//							
//							elementName = strBuild.toString();
//						}
		
						final IResource res = SDEditorManager.convertToIDEResource( element.eResource() );
						
						breakpoint = new MLSDMDebugBreakpoint( 	elementUUID, 
																elementName,
																element.eResource().getURI().toString(),
																"", 
																"OCL",
																res );
						DebugPlugin.getDefault().getBreakpointManager().addBreakpoint( breakpoint );
					}
					else {
						breakpoint.delete();
					}
				}
				catch ( final CoreException p_ex) {
					throw new ExecutionException( "Could not delete breakpoint of element '" + elementUUID + "'.", p_ex );
				}
			}
		}
		
		return null;
	}
	
    @Override
	public void dispose() {
    	emfLabelProvider.dispose();

    	super.dispose();
    }
}
