package de.mdelab.mlsdm.interpreter.debug.ui.mlsdmEditor;

import java.io.IOException;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.emf.common.ui.URIEditorInput;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.handlers.HandlerUtil;

import de.mdelab.instanceGraphEditor.ui.editor.InstanceGraphEditorInput;
import de.mdelab.mlcore.MLElementWithUUID;
import de.mdelab.mlsdm.interpreter.debug.ui.breakpoints.IMLSDMDebugUiBreakpointConstants;

/**
 * @see org.eclipse.core.commands.IHandler
 * @see org.eclipse.core.commands.AbstractHandler
 */
public class OpenInstanceGraphEditorHandler extends AbstractHandler {
	
    /**
	 * The constructor.
	 */
	public OpenInstanceGraphEditorHandler() {
		super();
	}

	@Override
	public Object execute( final ExecutionEvent p_event )
	throws ExecutionException {
		final ISelection currentSelection = HandlerUtil.getCurrentSelection( p_event );

		if ( currentSelection instanceof IStructuredSelection ) {
			final Object object = ( (IStructuredSelection) currentSelection ).getFirstElement();
			final EObject instanceObject;

			if ( object instanceof Resource ) {
				final Resource res = (Resource)  object;
				
				if ( res.getContents().isEmpty() ) {
					instanceObject = null;
				}
				else {
					instanceObject = ( (Resource) object ).getContents().get( 0 );
				}
			}
			else if ( object instanceof EObject ) {
				instanceObject = (EObject) object;
			}
			else {
				instanceObject = null;
			}
			
			if ( instanceObject != null ) {
				try {
					final IEditorInput editorInput;
					final String editorId;
					
					if ( instanceObject instanceof MLElementWithUUID ) {
						editorInput = new URIEditorInput( MLSDMEditorManager.getDiagramURI( (MLElementWithUUID) instanceObject ) );
						editorId = MLSDMDebuggerStoryDiagramEditor.ID;
					}
					else {
						editorInput = new InstanceGraphEditorInput( instanceObject.eResource(),  instanceObject );
						editorId = IMLSDMDebugUiBreakpointConstants.INSTANCE_GRAPH_EDITOR_ID;
					}
		
					final IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
				
					page.openEditor( editorInput, editorId );
				}
				catch ( final PartInitException p_ex ) {
					throw new ExecutionException( null, p_ex );
				}
				catch ( final IOException p_ex ) {
					throw new ExecutionException( null, p_ex );
				}
				catch ( final CoreException p_ex ) {
					throw new ExecutionException( null, p_ex );
				}
			}
		}	
		
		return null;
	}
}
